@@ -40,10 +40,12 @@ func Globals(vars map[string]interface{}) CompileOption {
 // IgnoreModule is an option for [NewCompiler] and [Compile] that allows
 // ignoring a given module.
 //
-// This option can be passed multiple times with different module names.
-// Alternatively, you can use [Compiler.IgnoreModule], but modules ignored this
-// way are not retained after [Compiler.Build] is called, unlike modules ignored
-// with the IgnoreModule option.
+// Import statements for ignored modules will be ignored without errors,
+// but a warning will be issued. Any rule that makes use of an ignored
+// module will be also ignored, while the rest of the rules that don't
+// rely on that module will be correctly compiled.
+//
+// This option can be used multiple times for ignoring different modules.
 func IgnoreModule(module string) CompileOption {
 	return func(c *Compiler) error {
 		c.ignoredModules[module] = true
@@ -126,6 +128,20 @@ func ErrorOnSlowLoop(yes bool) CompileOption {
 	}
 }
 
+// EnableIncludes allows the compiler to process include directives in YARA
+// rules. When this option is set to false, any include directive found in
+// the source code will be treated as an error. By default, includes are enabled.
+//
+// Example:
+//
+//	compiler, _ := yara_x.NewCompiler(yara_x.EnableIncludes(false))
+func EnableIncludes(yes bool) CompileOption {
+	return func(c *Compiler) error {
+		c.includesEnabled = yes
+		return nil
+	}
+}
+
 // A structure that contains the options passed to [Compiler.AddSource].
 type sourceOptions struct {
 	origin string
@@ -242,6 +258,7 @@ type Compiler struct {
 	conditionOptimization bool
 	errorOnSlowPattern    bool
 	errorOnSlowLoop       bool
+	includesEnabled       bool
 	ignoredModules        map[string]bool
 	bannedModules         map[string]bannedModule
 	vars                  map[string]interface{}
@@ -251,10 +268,11 @@ type Compiler struct {
 // NewCompiler creates a new compiler.
 func NewCompiler(opts ...CompileOption) (*Compiler, error) {
 	c := &Compiler{
-		ignoredModules: make(map[string]bool),
-		bannedModules:  make(map[string]bannedModule),
-		vars:           make(map[string]interface{}),
-		features:       make([]string, 0),
+		includesEnabled: true,
+		ignoredModules:  make(map[string]bool),
+		bannedModules:   make(map[string]bannedModule),
+		vars:            make(map[string]interface{}),
+		features:        make([]string, 0),
 	}
 
 	for _, opt := range opts {
@@ -280,6 +298,10 @@ func NewCompiler(opts ...CompileOption) (*Compiler, error) {
 		flags |= C.YRX_ERROR_ON_SLOW_LOOP
 	}
 
+	if !c.includesEnabled {
+		flags |= C.YRX_DISABLE_INCLUDES
+	}
+
 	C.yrx_compiler_create(flags, &c.cCompiler)
 
 	if err := c.initialize(); err != nil {
@@ -308,7 +330,7 @@ func (c *Compiler) initialize() error {
 	return nil
 }
 
-// AddSource adds some YARA source code to be compiled.
+// AddSource adds YARA source code to be compiled.
 //
 // This method may be invoked multiple times to add several sets of
 // YARA rules. If the rules provided in src contain errors that prevent
