@@ -29,14 +29,19 @@ pub use errors::Error;
 
 /// Abstract Syntax Tree (AST) for YARA rules.
 pub struct AST<'src> {
-    /// The list of imports.
-    pub imports: Vec<Import<'src>>,
-    /// The list of rules in the AST.
-    rules: Vec<Rule<'src>>,
+    /// The list of items in the AST (imports, includes, and rules).
+    pub items: Vec<Item<'src>>,
     /// Errors that occurred while parsing the rules.
     errors: Vec<Error>,
 }
 
+/// Top level items in the AST.
+pub enum Item<'src> {
+    Import(Import<'src>),
+    Include(Include<'src>),
+    Rule(Rule<'src>),
+}
+
 impl<'src> From<Parser<'src>> for AST<'src> {
     /// Crates an [`AST`] from the given parser.
     fn from(parser: Parser<'src>) -> Self {
@@ -45,16 +50,34 @@ impl<'src> From<Parser<'src>> for AST<'src> {
 }
 
 impl<'src> AST<'src> {
-    /// Returns the import statements in the AST.
+    /// Returns the top level items in the AST.
+    ///
+    /// A top level item can be an import, include, or rule.
     #[inline]
-    pub fn imports(&self) -> &[Import<'src>] {
-        self.imports.as_slice()
+    pub fn items(&self) -> impl Iterator<Item = &Item<'src>> {
+        self.items.iter()
+    }
+
+    /// Returns the import statements in the AST.
+    pub fn imports(&self) -> impl Iterator<Item = &Import<'src>> {
+        self.items.iter().filter_map(|item| {
+            if let Item::Import(import) = item {
+                Some(import)
+            } else {
+                None
+            }
+        })
     }
 
     /// Returns the rules in the AST.
-    #[inline]
-    pub fn rules(&self) -> &[Rule<'src>] {
-        self.rules.as_slice()
+    pub fn rules(&self) -> impl Iterator<Item = &Rule<'src>> {
+        self.items.iter().filter_map(|item| {
+            if let Item::Rule(rule) = item {
+                Some(rule)
+            } else {
+                None
+            }
+        })
     }
 
     /// Returns the errors found while parsing the source code.
@@ -73,7 +96,7 @@ impl<'src> AST<'src> {
 
 impl Debug for AST<'_> {
     fn fmt(&self, f: &mut Formatter<'_>) -> fmt::Result {
-        for rule in &self.rules {
+        for rule in self.rules() {
             write_tree(f, &ascii_tree::rule_ascii_tree(rule))?;
             writeln!(f)?;
         }
@@ -96,6 +119,13 @@ pub struct Import<'src> {
     pub module_name: &'src str,
 }
 
+/// An include statement.
+#[derive(Debug)]
+pub struct Include<'src> {
+    span: Span,
+    pub file_name: &'src str,
+}
+
 /// A YARA rule.
 #[derive(Debug)]
 pub struct Rule<'src> {
@@ -827,7 +857,8 @@ impl FuncCall<'_> {
     /// Span covered by the function's arguments in the source code.
     ///
     /// [`FuncCall::span`] covers the whole function call, including the
-    /// function identifier, while this covers only the arguments.
+    /// function identifier and the arguments, while this covers only the
+    /// arguments.
     pub fn args_span(&self) -> Span {
         self.args_span.clone()
     }
@@ -1122,6 +1153,12 @@ impl WithSpan for Import<'_> {
     }
 }
 
+impl WithSpan for Include<'_> {
+    fn span(&self) -> Span {
+        self.span.clone()
+    }
+}
+
 impl WithSpan for FuncCall<'_> {
     fn span(&self) -> Span {
         self.span.clone()
